/** @jsx createElement */
'use strict';
import {createElement, Component, PropTypes} from 'weex-rx';
import { TouchableHighlight, View, Text } from 'nuke-components';
//## todo
import {detect} from 'nuke-util';
const isWeex = typeof callNative !== 'undefined';
const isWeb = typeof window === 'object';
import rangStyles from '../style.rxscss';
import SwipeEvent from 'nuke-swipe';
import {trigger} from '../util.js';

const HANDLER_TYPES = {
    'handler':'nukeRange-handler',
    'tips'      :    'nukeRange-tips',
    'tipsContent'   :    'nukeRange-tips-content',
    'tapArea':'nukeRange-handler-tap'
}
class Handler extends Component {

    /**
     * 声明属性类型
     */
    static propTypes = {
        //是否为结束的handler
        isEnd: PropTypes.bool,
        position: PropTypes.number,
        onActive: PropTypes.func,
        onProcess: PropTypes.func,
        onChange: PropTypes.func,
        tips: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
        hasTips: PropTypes.bool
    };

    /**
     * 设置默认属性
     */
    static defaultProps = {
        position: 0
    };

    /**
     * 构造函数
     */
    constructor(props){
        super(props);
        this.trigger = trigger;
        this.active = false;
    }

    /**
     * 渲染: 当state发生改变时立即执行
     */
    render() {
        const {
            tips,
            position
            } = this.props;

        const baseStyle = rangStyles[HANDLER_TYPES['handler']];
        const style = {
            left: position + 'rem'
        }
        const handlerStyle = baseStyle;
        const tipsStyle = rangStyles[HANDLER_TYPES['tips']];
        const tipsContentStyle = rangStyles[HANDLER_TYPES['tipsContent']]
        let tapStyle = rangStyles[HANDLER_TYPES['tapArea']];
        tapStyle =  Object.assign({},tapStyle,style);

        //根据当前对Touch的支持情况, 绑定Touch的事件
        const events = detect.touchEvents;
        // const attrs = {
        //     [events.onTouchStart]: this.touchStartHandler,
        //     [events.onTouchEnd]: this.touchEndHandler,
        //     [events.onTouchMove]: this.touchMoveHandler,
            // [events.onTouchCancel]: this.touchCancelHandler
        // }


        return (
             <SwipeEvent
                onSwipeBegin={this.touchStartHandler.bind(this)}
                onSwipeEnd={this.touchEndHandler.bind(this)}
                onSwipe={this.touchMoveHandler.bind(this)}
                initialVelocityThreshold={0.01}
                vertical={false}
                horizontalThreshold={10}
                style={ tapStyle }
                >
                <View style={ handlerStyle }>
                    {this.renderTips(tipsStyle,tipsContentStyle)}
                </View>
             </SwipeEvent>
             )
    }

    renderTips(tipsStyle,tipsContentStyle){
        const {hasTips, tips} = this.props;

        if(!hasTips) return null;

        console.log('tips',tips);

        return (<View style={tipsStyle}>
                    <Text style={tipsContentStyle}>{tips}</Text>
                </View>);
        return null;
    }

    touchStartHandler({ direction, distance, velocity }){

        this.active = true;
        this.trigger('onActive',true,distance);
    }

    touchMoveHandler({ direction, distance, velocity }){
        //用户没有点上, 主要用于pc, 移动端move肯定是会touchStart的
        if(!this.active) return;

        console.warn('开始移动',direction,distance,velocity);
        this.trigger('onProcess', distance, this.props.isEnd);
    }

    touchEndHandler({ direction, distance, velocity }){
        // console.warn('结束',direction,distance,velocity);
        // this.active = false;
        // this.trigger('onChange', this.props.isEnd);
        // this.trigger('onActive', false);
        // this.trigger('onProcess', distance, this.props.isEnd);
    }

    getPosition(e){
        return typeof e.pageX === 'number' ? e.pageX : e.touches[0].pageX;
    }
}

export default Handler;
