/** @jsx createElement */
'use strict';
import {createElement, Component, PropTypes,findDOMNode} from 'weex-rx';
import { TouchableHighlight, View, Text } from 'nuke-components';
import Handler from './handler.js';
//## todo
import {detect} from 'nuke-util';
const isWeex = typeof callNative !== 'undefined';
const isWeb = typeof window === 'object';
import rangeStyles from '../style.rxscss';
import {trigger} from '../util.js';
import {string} from 'nuke-util';


const RANGE_TYPES = {
    'range'      :    'nukeRange',
    'bar'        :    'nukeRange-bar',
    'bar-after'  :    'nukeRange-bar-after',
    'barOverlay' :    'nukeRange-bar-overlay',
    'scaleLabel' :    'nukeRange-scale-label',
    'scaleMark'  :    'nukeRange-scale-mark',
    'mark-span' :     'nukeRange-scale-mark-item',
    'scale-span' :    'nukeRange-scale-label-item',
    'highlight'  :    'nukeRange-scale-mark-highlight'
};
/**
 * 声明属性类型
 */


class Range extends Component {
    getValuePropValue(child) {
        const props = child.props;
          if ('value' in props) {
            return props.value;
          }
          if ('defaultValue' in props) {
            return props.defaultValue;
          }
          return 0;
    }
    getPropValue(prop) {
      if (prop === 'value') {
        return this.getValuePropValue(this);
      }
      //没有的话则去defaultValue
      return this.props[prop];
    }

    /**
     * 构造函数
     */
    constructor(props){
        super(props);

        this.handlerProcessHandler = this.handlerProcessHandler.bind(this);
        this.handlerChangeHandler = this.handlerChangeHandler.bind(this);
        this.handlerActiveHandler = this.handlerActiveHandler.bind(this);
        this.getBarWidth = this.getBarWidth.bind(this);
        this.trigger = trigger;
        /**
         * 设置默认状态
         */
        const tmpWidth = rangeStyles[RANGE_TYPES['range']]['width'];
        this.state = {
            start: 0,
            end: 0,
            active: false,
            value: this.getPropValue('value'),
            barWidth:tmpWidth,
            left:0
        };
    }

    //收到新的props的操作, 可以在这里更新state
    componentWillReceiveProps (newProps){
        //示例, 当props发生改变后, 将会更新visible这个属性
        this.updateStateFromProps(newProps, ['value'], (newState) => {
            this.initialPosition();
        });
    }

    componentDidMount(){
        this.marks = this.getMarks();
        this.initialPosition();
    }

    /**
     * 渲染: 当state发生改变时立即执行
     */
    render() {
        const {
            prefix = this.defaultPrefix,
            style,
            disabled,
            ...others } = this.props;

        const baseStyle = rangeStyles[RANGE_TYPES['range']];
        let rangeStyle = Object.assign({},style,baseStyle,{active: this.state.active,
            disabled: disabled})
        return (<View style={rangeStyle} {...others}>
            {this.renderScale(true)}
            {this.renderBar()}
        </View>);
    }

    renderBar(){
        const {start, end} = this.state;
        const baseStyle = rangeStyles[RANGE_TYPES['barOverlay']];
        let overlayStyle = Object.assign({}, baseStyle,{
            marginLeft: start + 'rem',
            width: (end-start) + 'rem'
        });

        const barStyle = Object.assign({},rangeStyles[RANGE_TYPES['bar']]);
        const barStyleAfter =Object.assign(rangeStyles[RANGE_TYPES['bar-after']],{width:this.getBarWidth()});
        return <View style={ barStyle } ref="bar">
            <View style={ barStyleAfter }></View>
            <View style={ overlayStyle } />
            {this.renderScale(false)}
            {this.renderHandler(false)}
            {this.renderHandler(true)}
            </View>
    }

    renderHandler(isEnd){
        //single的话情况, 不显示start的handler
        if(!isEnd && this.props.slider === 'single') return null;

        const {start, end} = this.state;
        const value = isEnd ? end : start;

        return (
            <Handler
                isEnd={isEnd}
                hasTips={this.props.hasTips}
                onProcess={this.handlerProcessHandler}
                onChange={this.handlerChangeHandler}
                onActive={this.handlerActiveHandler}
                tips={this.getTips(this.pixelToValue(value, isEnd))}
                position={value}/>)
    }

    renderScale(isLabel){
        if(!this.marks) return null;
        let scaleStyle = isLabel ? rangeStyles[RANGE_TYPES['scaleLabel']]:rangeStyles[RANGE_TYPES['scaleMark']]
        return (<View style={ scaleStyle }>
            {this.renderMarkItem(isLabel)}
        </View>);
    }

    renderMarkItem(isLabel){
        let result = [];
        const highlight = rangeStyles[RANGE_TYPES['highlight']];
        for(let i = 0; i < this.marks.length; i ++){
            const current = this.marks[i];
            const temPercent = current.percent/100;
            var currentPX = this.getBarWidth()*temPercent;

            const style = {left: currentPX+'rem'};
            const labelCorrect = {marginLeft:'-50rem',width:'100rem'};

            const spanStyle = rangeStyles[RANGE_TYPES['scale-span']];
            const markSpan = rangeStyles[RANGE_TYPES['mark-span']];
            let tmpStyle = isLabel ? spanStyle : markSpan;
            let itemStyle =  (!isLabel && this.markIsHighlight(current.value))
                        ?
                 Object.assign({},style,tmpStyle,highlight)
                        :
                 Object.assign({},style,highlight,tmpStyle)

            isLabel ? itemStyle =  Object.assign({},style,highlight,tmpStyle,labelCorrect) : null;

            // const itemStyle = this.markIsHighlight(current.value) ? Object.assign({},style,tmpStyle) : null;
            const text = isLabel ? current.label : null;
            result.push(<Text key={i} style={itemStyle}>{text}</Text>);
        }

        return result;
    }

    //检查mark是否需要highlight
    markIsHighlight(value){
        const position = this.valueToPixel(value);
        return position >= this.state.start && position <= this.state.end;
    }

    getMarks(){
        let {marks, min, max} = this.props;
        if(!marks) return;

        //非数组的话, 则均分
        marks = typeof(marks) === 'number' ? this.equalDivisionMarks(marks) : marks;
        let result = [];

        for(let value in marks){
            const label = marks[value];
            value = parseInt(value);
            const percent = parseInt(value / (max - min) * 100);
            result.push({
                value: value,
                percent: percent,
                label: label
            });
        }

        return result;
    }

    //根据值均分marks
    equalDivisionMarks(value){
        const {min, max} = this.props;
        const total = Math.ceil((max - min) / (value));

        let result = {};
        for(let i = 0; i <= total; i++){
            const current = parseInt(i * value);
            result[current] = current;
        }

        return result;
    }


    //初始化位置
    initialPosition() {
        const barWidth =this.getBarWidth();
        let {min, max} = this.props;
        let value = this.getPropValue('value');
        let ratio = barWidth / (max - min);

        if (typeof(value) === 'number') value = [0, value];
        const [startValue, endValue] = value;
        const start = Math.max(0, startValue * ratio);
        const end = Math.min(endValue * ratio, barWidth);

        this.setState({
            start: start,
            end: end
        });
    }

    //计算handler所在的位置, 以及bar的位置
    calculatePosition(x, isEndHandler){

        const {min, max, step} = this.props;
        const barWidth = this.getBarWidth();
        // const barElement = findDOMNode(this.refs.bar);
        // const start = barElement.offsetLeft;
        let position = 0;
        if(isEndHandler){
            if(isWeex)
                position = x + this.state.left;
            else
                position = x*2 + this.state.left;
        }
        else{
            if(isWeex)
                position = x + this.state.right;
            else
                position = x*2 + this.state.right;
        }
        if(step > 0){
            //每一步的长度
            const valuePerStep = barWidth / (max - min) * step;
            //总共有多少步
            let totalStep =  position / valuePerStep;
            totalStep = isEndHandler ? Math.round(totalStep) : Math.floor(totalStep);
            //根据step计算应当所在的位置,
            position = totalStep * valuePerStep;
        }

        //结束的handler, 不能大于bar的宽度, 也不能小于start的位置
        if(isEndHandler){
            return Math.max(Math.min(barWidth, position), this.state.start);
        }else{
            //开始的handler, 不能大于end的位置, 也不能小于0
            return Math.max(Math.min(this.state.end, position), 0);
        }
    }

    getBarWidth(){
        const {style} = this.props;
        const {barWidth}=this.state;

        let remWidth ;
        if(style && style.width)
            remWidth = style.width;
        else
            remWidth = barWidth;
        //目前经支持rem单位
        return remWidth.split('rem')[0]-20 || 0;
    }

    //将value转换为像素
    valueToPixel(value){
        const {min, max} = this.props;
        const barWidth = this.getBarWidth();
        return barWidth / (max - min) * value;
    }

    getTips(value){
        const {formatter} = this.props;
        return (formatter && string.format(formatter, value)) || value;
    }

    //将pixel转换为value
    pixelToValue(pixel, isEnd){
        const {min, max} = this.props;
        const barWidth = this.getBarWidth();
        let value = (max - min) / barWidth * pixel;
        value = isEnd ? Math.ceil(value) : Math.floor(value);
        return Math.min(Math.max(value, min), max);
    }

    //获取Trigger需要的value
    getTriggerValue(data){
        //触发事件
        const startValue = this.pixelToValue(data.start);
        const endValue = this.pixelToValue(data.end, true);
        return this.props.slider === 'single' ? endValue : [startValue, endValue];
    }

    //======================================== 事件 ========================================
    handlerActiveHandler(active,distance){
        console.log('start');
        if(this.props.disabled) return;
        this.setState({
            active: active,
            left:this.state.end,
            right:this.state.start
        });
    }

    handlerChangeHandler(isEndHandler){
        if(this.props.disabled) return;
        //触发事件
        this.trigger('onChange', this.getTriggerValue(this.state));
    }

     /*
     * 检查某个key是否为非受控的组件, 只要值为undefined, 则认为是非受控组件
     * 提取这个检测方式, 是因为某些时候, 组件需要直接检查是否非受控, 则不是使用setUnControlledState
     * @param {string} key - 要检查的键值
     * @param {any} value - 要更新的值
     */
    isUnControlledState(key){
        return this.props[key] === undefined;
    }

    //handler发生改变动的事件
    handlerProcessHandler(x, isEndHandler){

        if(this.props.disabled) return;

        const {start, end} = this.state;
        const position = this.calculatePosition(x, isEndHandler);
        const newState = {
            start: start,
            end: end
        };

        newState[isEndHandler ? 'end' : 'start'] = position;
        //避免多次重复渲染
        if(newState.start === start && newState.end === end) return;

        //检查是否为非受控组件, 非受控组件才更新state
        if(this.isUnControlledState('value')){
            this.setState(newState);
        }

        //触发事件
        this.trigger('onProcess', this.getTriggerValue(newState));
    }
}
Range.propTypes =  {
    slider: PropTypes.oneOf(['single', 'double']),
    min: PropTypes.number,
    max: PropTypes.number,
    step: PropTypes.number,
    value: PropTypes.oneOfType([PropTypes.number, PropTypes.array]),
    defaultValue: PropTypes.oneOfType([PropTypes.number, PropTypes.array]),
    marks: PropTypes.oneOfType([PropTypes.number, PropTypes.object, PropTypes.boolean]),
    disabled: PropTypes.bool,
    onChange: PropTypes.func,
    onProcess: PropTypes.func,
    hasTips: PropTypes.bool,
    formatter: PropTypes.string
},
 /**
 * 设置默认属性
 */
Range.defaultProps = {
    min: 0,
    max: 100,
    step: 0,
    slider: 'single'
}
//提供给Form的识别参数
Range.formControl = true;
Range.controlledTrigger = 'onProcess';
export default Range;
